@@ -1,25 +1,31 @@
 #!/usr/bin/env python
 
-import md5, os, stat
+import md5, os, stat, zlib
 
 READSIZE=8192
 
-def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE):
+def md5it(f):
+    h = md5.new()
+    d = f.read(READSIZE)
+    while d:
+        h.update(d)
+        d = f.read(READSIZE)
+    return h.hexdigest()
+
+def headadler8bit(f):
+    adlerval = 0
+    data = f.read(READSIZE)
+    adlerval = zlib.adler32(data, adlerval)
+    return adlerval % 256
+
+def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE, PROFILES):
     assert not (HARDLINKEM and DELETEEM) # at most one of these
     # d1 = { k: size, v: { k: (dev, ino,), v: [ fname ] } }
     d1 = {}
 
     visiteddirs = set() # to avoid symlink recursion
 
-    def md5it(f):
-        h = md5.new()
-        d = f.read(READSIZE)
-        while d:
-            h.update(d)
-            d = f.read(READSIZE)
-        return h.hexdigest()
-
-    # First pass: build map from size, dev, inode number to filenames.
+    # First pass: build map from size, dev, inode number to filenames, and if PROFILES then print out the headadler8bit and size of each file.
     def visit(dirpath, dirnames, fnames):
         if dirpath in visiteddirs:
             return
@@ -35,11 +41,17 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE):
                 thestat = os.stat(fullfname)
                 if not stat.S_ISLNK(thestat.st_mode) and thestat.st_size >= MINSIZE:
                     d2 = d1.setdefault(thestat.st_size, {})
-                    l = d2.setdefault((thestat.st_dev, thestat.st_ino,), [])
+                    if d2.has_key((thestat.st_dev, thestat.st_ino,)):
+                        l = d2[(thestat.st_dev, thestat.st_ino,)]
+                    else:
+                        l = []
+                        d2[(thestat.st_dev, thestat.st_ino,)] = l
+                        if PROFILES:
+                            print "%x %s" % (headadler8bit(open(fullfname, 'r')), thestat.st_size)
                     l.append(fullfname)
-            except:
+            except EnvironmentError, le:
                 if VERBOSITY:
-                    print "note: couldn't stat file named %s" % (fullfname,)
+                    print "note: couldn't stat file named %s because %s" % (fullfname, le)
         for dirname in dirnames:
             if VERBOSITY > 2:
                 print "about to check islink(%s): %s" % (dirname, os.path.islink(dirname),)
@@ -66,9 +78,8 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE):
                     try:
                         md5h = md5it(open(fname, "r")) # Here's the expensive part.
                         break
-                    except:
-                        print "note: couldn't read file named %s" % (fname,)
-                        raise
+                    except EnvironmentError, le:
+                        print "note: couldn't read file named %s because %s" % (fname, le)
 
                 if md5h is None and len(fnames) > 1:
                     print "note: couldn't read any of these (same-inode) files: %s" % (fnames,)
@@ -82,7 +93,10 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE):
                     if HARDLINKEM or DELETEEM:
                         onename = d4.items()[0][1][0]
                     for ((iterdev, iterino,), iterfnames,) in d4.iteritems():
-                        print "md5: %s, dev: %s, ino: %s, size: %s, fnames: %s" % (md5h, iterdev, iterino, size, iterfnames,)
+                        if PROFILES:
+                            print "identical; headadler8bit: %x, dev: %s, ino: %s, size: %s" % (headadler8bit(open(onename, 'r')), iterdev, iterino, size,)
+                        else:
+                            print "md5: %s, dev: %s, ino: %s, size: %s, fnames: %s" % (md5h, iterdev, iterino, size, iterfnames,)
                         if HARDLINKEM or DELETEEM:
                             for fname in iterfnames:
                                 if fname != onename:
@@ -94,7 +108,10 @@ def dffem(VERBOSITY, IGNOREDIRS, HARDLINKEM, DELETEEM, MINSIZE):
                                         print "deleting %s (since there is already %s)" % (fname, onename)
                 elif len(d4) > 2:
                     # More collisions.  Print out the new one.
-                    print "md5: %s, dev: %s, ino: %s, size: %s, fnames: %s" % (md5h, dev, ino, size, fnames,)
+                    if PROFILES:
+                        print "identical; headadler8bit: %x, dev: %s, ino: %s, size: %s" % (headadler8bit(open(fnames[0], 'r')), dev, ino, size,)
+                    else:
+                        print "md5: %s, dev: %s, ino: %s, size: %s, fnames: %s" % (md5h, dev, ino, size, fnames,)
                     if HARDLINKEM or DELETEEM:
                         for fname in fnames:
                             if fname != onename:
